#include <confuse.h> /* header for libconfuse */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <apr-1/apr.h>
#include <apr-1/apr_pools.h>
#include <apr-1/apr_strings.h>

#include "ganglia.h"
#include "send_gmetric.h"
#include "trace.h"

Ganglia_pool global_context;
Ganglia_gmond_config gmond_config;
Ganglia_udp_send_channels send_channels;

int init_gmetric(const char* conf) {
  char* conf_local = (char*)conf;

  /* create the global context */
  global_context = Ganglia_pool_create(NULL);
  if (!global_context) {
    TRACE(0, "Unable to create global context. Exiting.");

    return 1;
  }
  /* parse the configuration file */
  int use_default_config = 1;
  if (strcmp(conf, "") != 0) {
    use_default_config = 0;
  }
  gmond_config = Ganglia_gmond_config_create(conf_local, use_default_config);
  if (!gmond_config) {
    TRACE(0, "Unable to load Ganglia configuration. Exiting.");
    return 1;
  }

  /* build the udp send channels */
  send_channels = Ganglia_udp_send_channels_create(global_context, gmond_config);
  if (!send_channels) {
    TRACE(0, "Unable to create ganglia send channels. Exiting.");
    return 1;
  }

  return 0;
}

void destroy_gmetric() { Ganglia_pool_destroy(global_context); }

int send_gmetric(const char* name, const char* value, const char* type, const char* units, const char* slope, int tmax,
                 int dmax, const char* group, const char* cluster, const char* desc, const char* title) {
  Ganglia_metric gmetric;

  int rval;

  /* Create local char* versions of the parameters */
  char* name_local = (char*)name;
  char* value_local = (char*)value;
  char* type_local = (char*)type;
  char* units_local = (char*)units;
  char* slope_local = (char*)slope;
  char* group_local = (char*)group;
  char* cluster_local = (char*)cluster;
  char* desc_local = (char*)desc;
  char* title_local = (char*)title;

  /* create the message */
  gmetric = Ganglia_metric_create(global_context);
  if (!gmetric) {
    TRACE(0, "Unable to allocate gmetric structure. Exiting.");
    return 1;
  }
  // apr_pool_t *gm_pool = (apr_pool_t*)gmetric->pool;

  if (!(strcmp(name, "") != 0 && strcmp(value, "") != 0 && strcmp(type, "") != 0)) {
    TRACE(0, "Incorrect options supplied, exiting.\n");
    return 1;
  }
  rval = Ganglia_metric_set(gmetric, name_local, value_local, type_local, units_local, cstr_to_slope(slope_local), tmax,
                            dmax);

  /* TODO: make this less ugly later */
  switch (rval) {
    case 1:
      TRACE(0, "gmetric parameters invalid. exiting.\n");
      return 1;
    case 2:
      TRACE(0, "one of your parameters has an invalid character '\"'. exiting.\n");
      return 1;
    case 3:
      TRACE(0, "the type parameter \"%s\" is not a valid type. exiting.\n", type);
      return 1;
    case 4:
      TRACE(0, "the value parameter \"%s\" does not represent a number. exiting.\n", value);
      return 1;
  }

  if (strcmp(cluster, "") != 0) Ganglia_metadata_add(gmetric, "CLUSTER", cluster_local);
  if (strcmp(group, "") != 0) {
    char* last;
    for (char* groupArg = apr_strtok(group_local, ", ", &last); groupArg != NULL;
         groupArg = apr_strtok(NULL, ", ", &last)) {
      Ganglia_metadata_add(gmetric, "GROUP", groupArg);
    }
  }
  if (strcmp(desc, "") != 0) Ganglia_metadata_add(gmetric, "DESC", desc_local);
  if (strcmp(title, "") != 0) Ganglia_metadata_add(gmetric, "TITLE", title_local);

  /* send the message */
  rval = Ganglia_metric_send(gmetric, send_channels);
  if (rval) {
    TRACE(0, "There was an error sending to %d of the send channels.\n", rval);
  }

  /* cleanup */
  //	Ganglia_udp_send_channels_destroy(send_channels);
  //	Ganglia_gmond_config_destroy(gmond_config);
  Ganglia_metric_destroy(gmetric); /* not really necessary but for symmetry */

  return rval;
}
